#include "Cubemap.h"
#include "Texture.h"
#include <cmath>
#include <fstream>
#include "rgbe.h"

using namespace std;

bool dummyTGAHeader(ifstream &currentfile, int &sizeX, int &sizeY)
{
	char dummy;
	char temp;
	currentfile.get(dummy).get(dummy);
	currentfile.get(temp);                  /* RGB non compresse */
	if (temp!=2)
		return false;
	currentfile.get(dummy).get(dummy);
	currentfile.get(dummy).get(dummy);
	currentfile.get(dummy);
	currentfile.get(dummy).get(dummy);           /* origine X */
	currentfile.get(dummy).get(dummy);           /* origine Y */
	currentfile.get(temp);
	sizeX = temp;
	currentfile.get(temp);
	sizeX += temp * 256;

	currentfile.get(temp);
	sizeY = temp;
	currentfile.get(temp);
	sizeY += temp * 256;

    currentfile.get(temp);                 /* 24 bit bitmap */
	currentfile.get(dummy);
	return true;
}

bool cubemap::Init()
{
    if (texture)
    {
        return false;
    }
    ifstream currentfile ;
    color * currentColor;
    int x,y, dummySizeX, dummySizeY;

    currentfile.open(name[up].c_str(), ios_base::binary);
    if ((!currentfile)||(!dummyTGAHeader(currentfile, sizeX, sizeY)))
        return false;
    if (sizeX <= 0 || sizeY <= 0)
        return false;
    texture = new color[sizeX * sizeY * 6];

    int number = cubemap::up * sizeX * sizeY;
    currentColor = 	texture + number;
    for (y = 0; y < sizeY; y++)
    for (x = 0; x < sizeX; x++)
    {
        currentColor->blue = currentfile.get() /255.0f;
        currentColor->green = currentfile.get() /255.0f;
        currentColor->red = currentfile.get() /255.0f;
        currentColor++;
    }
    currentfile.close();
	
    for (unsigned i = cubemap::down; i <= cubemap::backward; ++i)
    {
        number = i * sizeX * sizeY;
        currentColor = 	texture + number;
        currentfile.open(name[i].c_str(), ios_base::binary);
        if ((!currentfile)||
            (!dummyTGAHeader(currentfile, dummySizeX, dummySizeY)) ||
            sizeX != dummySizeX || 
            sizeY != dummySizeY)
        {
            // Les textures doivent tre de taille identique..
            delete [] texture;
            texture = 0;
            return false;
        }

        for (y = 0; y < sizeY; y++)
        for (x = 0; x < sizeX; x++)
        {
            currentColor->blue = currentfile.get() /255.0f;
            currentColor->green = currentfile.get() /255.0f;
            currentColor->red = currentfile.get() /255.0f;
            currentColor++;
        }
        currentfile.close();
    }

    return true;
}

bool cubemap::InitHDR()
{
    if (texture)
    {
        return false;
    }
    ifstream currentfile ;
    color * currentColor;
    int dummySizeX, dummySizeY;
    rgbe_header_info info;

    currentfile.open(name[up].c_str(), ios_base::binary);
    if ((!currentfile)||(!RGBE_ReadHeader(currentfile, &sizeX, &sizeY, &info)))
        return false;
    if (sizeX <= 0 || sizeY <= 0)
        return false;
    texture = new color[sizeX * sizeY * 6];

    int number = cubemap::up * sizeX * sizeY;
    currentColor = 	texture + number;

    RGBE_ReadPixels_RLE(currentfile, currentColor, sizeX, sizeY);

    currentfile.close();
	
    for (unsigned i = cubemap::down; i <= cubemap::backward; ++i)
    {
        number = i * sizeX * sizeY;
        currentColor = 	texture + number;
        currentfile.open(name[i].c_str(), ios_base::binary);
        if ((!currentfile)||
            (!RGBE_ReadHeader(currentfile, &dummySizeX, &dummySizeY, &info)) ||
            sizeX != dummySizeX || 
            sizeY != dummySizeY)
        {
            // Les textures doivent tre de taille identique..
            delete [] texture;
            texture = 0;
            return false;
        }

        RGBE_ReadPixels_RLE(currentfile, currentColor, sizeX, sizeY);

        currentfile.close();
    }

    return true;
}

color readCubemap(const cubemap & cm, ray myRay)
{
    color * currentColor ;
    color outputColor = {0.0f,0.0f,0.0f};
    if(!cm.texture)
    {
        return outputColor;
    }
    if ((fabsf(myRay.dir.x) >= fabsf(myRay.dir.y)) && (fabsf(myRay.dir.x) >= fabsf(myRay.dir.z)))
    {
        if (myRay.dir.x > 0.0f)
        {
            currentColor = cm.texture + cubemap::right * cm.sizeX * cm.sizeY;
            outputColor = readTexture(currentColor,  
                1.0f - (myRay.dir.z / myRay.dir.x+ 1.0f) * 0.5f,  
                (myRay.dir.y / myRay.dir.x+ 1.0f) * 0.5f, cm.sizeX, cm.sizeY);
        }
        else if (myRay.dir.x < 0.0f)
        {
            currentColor = cm.texture + cubemap::left * cm.sizeX * cm.sizeY;
            outputColor = readTexture(currentColor,  
                1.0f - (myRay.dir.z / myRay.dir.x+ 1.0f) * 0.5f,
                1.0f - ( myRay.dir.y / myRay.dir.x + 1.0f) * 0.5f,  
                cm.sizeX, cm.sizeY);
        }
    }
    else if ((fabsf(myRay.dir.y) >= fabsf(myRay.dir.x)) && (fabsf(myRay.dir.y) >= fabsf(myRay.dir.z)))
    {
        if (myRay.dir.y > 0.0f)
        {
            currentColor = cm.texture + cubemap::up * cm.sizeX * cm.sizeY;
            outputColor = readTexture(currentColor,  
                (myRay.dir.x / myRay.dir.y + 1.0f) * 0.5f,
                1.0f - (myRay.dir.z/ myRay.dir.y + 1.0f) * 0.5f, cm.sizeX, cm.sizeY);
        }
        else if (myRay.dir.y < 0.0f)
        {
            currentColor = cm.texture + cubemap::down * cm.sizeX * cm.sizeY;
            outputColor = readTexture(currentColor,  
                1.0f - (myRay.dir.x / myRay.dir.y + 1.0f) * 0.5f,  
                (myRay.dir.z/myRay.dir.y + 1.0f) * 0.5f, cm.sizeX, cm.sizeY);
        }
    }
    else if ((fabsf(myRay.dir.z) >= fabsf(myRay.dir.x)) && (fabsf(myRay.dir.z) >= fabsf(myRay.dir.y)))
    {
        if (myRay.dir.z > 0.0f)
        {
            currentColor = cm.texture + cubemap::forward * cm.sizeX * cm.sizeY;
            outputColor = readTexture(currentColor,  
                (myRay.dir.x / myRay.dir.z + 1.0f) * 0.5f,  
                (myRay.dir.y/myRay.dir.z + 1.0f) * 0.5f, cm.sizeX, cm.sizeY);
        }
        else if (myRay.dir.z < 0.0f)
        {
            currentColor = cm.texture + cubemap::backward * cm.sizeX * cm.sizeY;
            outputColor = readTexture(currentColor,  
                (myRay.dir.x / myRay.dir.z + 1.0f) * 0.5f,  
                1.0f - (myRay.dir.y /myRay.dir.z+1) * 0.5f, cm.sizeX, cm.sizeY);
        }
    }
    if (cm.bsRGB)
    {
       // On s'assure que les donnes sont ramenes dans un espace
       // de reprsentation linaire au pralable
       outputColor.blue   = powf(outputColor.blue, 2.2f);
       outputColor.red    = powf(outputColor.red, 2.2f);
       outputColor.green  = powf(outputColor.green, 2.2f);
    }

    if (cm.bExposed)
    {
        // Les images LDR ont dj t soumis  la fonction d'exposition
        // on les ramne  un espace similaire  l'espace de dpart.
        outputColor.blue  = -logf(1.001f - outputColor.blue);
        outputColor.red   = -logf(1.001f - outputColor.red);
        outputColor.green = -logf(1.001f - outputColor.green);
    }

    outputColor.blue  /= cm.exposure;
    outputColor.red   /= cm.exposure;
    outputColor.green /= cm.exposure;

    return outputColor;
}
