#pragma once
#ifndef __RAYTRACE_H
#define __RAYTRACE_H
#include "Def.h"
#include "Texture.h"
#include "Cubemap.h"
#include "Blob.h"
#include <vector>

struct materialChannel {
    float diffuse;
    //Deuxime couleur diffuse optionnelle pour les matriaux procduraux
    float diffuse2; 
    float reflection, refraction, density, absorption;
    float specular;
    float power;
};

struct material {
    enum {
        gouraud=0,
        noise=1,
        marble=2,
        turbulence=3
    } type;

    float bump;

    materialChannel tab[COLOROFFSET_MAX];

    materialChannel & getChannel(COLOROFFSET offset) { return tab[int(offset)]; }
    const materialChannel & getChannel(COLOROFFSET offset) const { return tab[int(offset)]; }
};

struct light {
	point pos;
	color intensity;
};

struct perspective {
    enum {
        orthogonal = 0,
        conic = 1
    } type;

    float FOV;
    float clearPoint;
    float dispersion;
};

struct scene {
    std::vector<material> matTab;
	std::vector<sphere>   sphTab;
	std::vector<blob>     blbTab;
	std::vector<light>    lgtTab;
	int sizex, sizey;
    enum {
        high, 
        medium, 
        low
    } supersampleType;
    cubemap               cm;
    perspective           persp;
    int                   complexity;
};

struct context {
    COLOROFFSET offset;
    float fRefractionCoef;
    color cLightScattering;
    int   level;
    float absorption;

    static const context & getDefaultAir() 
    {
        static context airContext = {COLOROFFSET_GREEN, 1.0f, {0.0f,0.0f,0.0f}, 0, 1.0f};
        return airContext;
    };
};

#define invsqrtf(x) (1.0f / sqrtf(x))

#endif // __RAYTRACE_H
