#ifndef _H_RGBE
#define _H_RGBE

// le code de rgbe.h et rgbe.cpp viennent du site
// de Stanford. Je ne commente pas, il y a deja des commentaires
// en anglais.

#pragma once

struct rgbe_header_info {
  int valid;            /* indicate which fields are valid */
  char programtype[16]; /* listed at beginning of file to identify it 
                         * after "#?".  defaults to "RGBE" */ 
  float gamma;          /* image has already been gamma corrected with 
                         * given gamma.  defaults to 1.0 (no correction) */
  float exposure;       /* a value of 1.0 in an image corresponds to
			 * <exposure> watts/steradian/m^2. 
			 * defaults to 1.0 */
} ;

/* flags indicating which fields in an rgbe_header_info are valid */
enum RGBE 
{
 RGBE_VALID_PROGRAMTYPE = 0x01,
 RGBE_VALID_GAMMA = 0x02,
 RGBE_VALID_EXPOSURE = 0x04
};

#include <fstream>

struct color;

/* read or write headers */
/* you may set rgbe_header_info to null if you want to */
bool RGBE_ReadHeader(std::ifstream &currentfile, int *width, int *height, rgbe_header_info *info);

/* read or write pixels */
/* can read or write pixels in chunks of any size including single pixels*/
bool RGBE_ReadPixels(std::ifstream &currentfile, color *data, int numpixels);

/* read or write run length encoded files */
/* must be called to read or write whole scanlines */
bool RGBE_ReadPixels_RLE(std::ifstream &currentfile, color *data, int scanline_width,
			int num_scanlines);

#endif /* _H_RGBE */




